@@ -4,12 +4,14 @@ Lager CLI
 A Command Line Interface for Lager Data
 """
 
-__version__ = '0.1.95'
+__version__ = '0.1.104'
 
 SUPPORTED_DEVICES = (
     'at91samdexx',
     'at91samdgxx',
     'at91samdxx',
+    'at91sam4sxx',
+    'at91sam4sd32x',
     'atsame70',
     'cc3220s',
     'cc3220sf',
@@ -42,6 +44,7 @@ SUPPORTED_DEVICES = (
     'MIMXRT1064xxx6A',
     'lpc54xx',
     'lpc55xx',
+    'pic32mm',
 )  # Empty docstring below is to suppress tuple docstring that would otherwise be printed by autodoc
 """
 """
