@@ -53,7 +53,7 @@ def print_docker_error(ctx, error):
     stdout = parsed['stdout']
     stderr = parsed['stderr']
     click.echo(stdout, nl=False)
-    click.echo(stderr, err=True, nl=False)
+    click.secho(stderr, fg='red', err=True, nl=False)
     ctx.exit(parsed['returncode'])
 
 def print_canbus_error(ctx, error):
@@ -116,7 +116,7 @@ class LagerSession(BaseUrlSession):
 
             if error['code'] in OPENOCD_ERROR_CODES:
                 print_openocd_error(error['description'])
-            elif error['code'] in DOCKER_ERROR_CODES:
+            elif error['code'] in DOCKER_ERROR_CODES or error['code'] == 'wifi_connection_failed':
                 print_docker_error(ctx, error['description'])
             elif error['code'] in CANBUS_ERROR_CODES:
                 print_canbus_error(ctx, error['description'])
@@ -239,12 +239,19 @@ class LagerSession(BaseUrlSession):
         url = 'gateway/{}/run-python'.format(quote(gateway))
         return self.post(url, files=files, stream=True)
 
-    def kill_python(self, gateway, sig=signal.SIGTERM):
+    def run_pip(self, gateway, args):
+        """
+            Run python on a gateway
+        """
+        url = 'gateway/{}/run-pip'.format(quote(gateway))
+        return self.post(url, json={'args': args}, stream=True)
+
+    def kill_python(self, gateway, lager_process_id, sig=signal.SIGTERM):
         """
             Run python on a gateway
         """
         url = 'gateway/{}/kill-python'.format(quote(gateway))
-        return self.post(url, json={'signal': sig})
+        return self.post(url, json={'signal': sig, 'lager_process_id': lager_process_id})
 
     def gateway_hello(self, gateway):
         """
@@ -309,6 +316,19 @@ class LagerSession(BaseUrlSession):
         }
         return self.post(url, json=json_data)
 
+    def remote_debug(self, gateway, archive, elf_file, interpreter):
+        """
+            Start a remote debugging session
+        """
+        url = 'gateway/{}/remote-debug'.format(quote(gateway))
+
+        files = [
+            ('elf_file', elf_file),
+            ('archive', archive),
+            ('interpreter', interpreter),
+        ]
+        return self.post(url, files=files)
+
     def rename_gateway(self, gateway, new_name):
         """
             Rename a gateway
