@@ -14,12 +14,13 @@ def gpio():
     pass
 
 _GPIO_CHOICES = click.IntRange(0, 14)
-_LEVEL_CHOICES = click.Choice(('LOW', 'HIGH'), case_sensitive=False)
+_LEVEL_CHOICES = click.Choice(('LOW', 'HIGH', 'ON', 'OFF'), case_sensitive=False)
+_BUS_CHOICES = click.Choice(['5V', '3V3'], case_sensitive=False)
 
 @gpio.command(name='set')
 @click.option('--gateway', required=False, help='ID of gateway to which DUT is connected')
 @click.argument('gpio_', metavar='GPIO', type=_GPIO_CHOICES)
-@click.argument('type_', type=click.Choice(('IN', 'OUT'), case_sensitive=False))
+@click.argument('type_', type=click.Choice(('IN', 'OUT', 'ALT0', 'ALT1', 'ALT2', 'ALT3', 'ALT4', 'ALT5'), case_sensitive=False))
 @click.option('--pull', type=click.Choice(('UP', 'DOWN', 'OFF'), case_sensitive=False), default='OFF', show_default=True, help='Sets or clears the internal GPIO pull-up/down resistor.')
 @click.pass_context
 def set_(ctx, gateway, gpio_, type_, pull):
@@ -34,7 +35,7 @@ def set_(ctx, gateway, gpio_, type_, pull):
     if gateway is None:
         gateway = get_default_gateway(ctx)
 
-    if type_ == 'OUT' and pull != 'OFF':
+    if type_ != 'IN' and pull != 'OFF':
         click.echo(f'GPIO pin {gpio_} set as output, ignoring --pull', err=True)
 
     ctx.obj.session.gpio_set(gateway, gpio_, type_, pull)
@@ -88,3 +89,19 @@ def trigger(ctx, gateway, gpio_, pulse_length, level):
     if gateway is None:
         gateway = get_default_gateway(ctx)
     ctx.obj.session.gpio_trigger(gateway, gpio_, pulse_length, level)
+
+
+@gpio.command()
+@click.option('--gateway', required=False, help='ID of gateway to which DUT is connected')
+@click.argument('bus_', metavar='POWER_BUS', type=_BUS_CHOICES)
+@click.argument('level', type=_LEVEL_CHOICES)
+@click.pass_context
+def power(ctx, gateway, bus_, level):
+    """
+        Sets level on power bus enable lines
+
+    """
+    if gateway is None:
+        gateway = get_default_gateway(ctx)
+    ctx.obj.session.gpio_power(gateway, bus_, level)
+
