@@ -7,6 +7,7 @@ try:
     _TERMIOS_IMPORT_FAILED = False
     import termios
     import tty
+    import pty
 except (ImportError, ModuleNotFoundError) as exc:
     _TERMIOS_IMPORT_FAILED = exc
 import platform
@@ -25,6 +26,7 @@ import wsproto.frame_protocol as wsframeproto
 import requests
 from .matchers import test_matcher_factory
 from .util import heartbeat
+from .simple_hdlc import HDLC
 
 def stream_response(response):
     """
@@ -129,13 +131,20 @@ class TTYIO:
     """
         Class for doing I/O with a TTY managed by curses
     """
-    def __init__(self, line_ending=b'', opost=False):
+    def __init__(self, line_ending=b'', opost=False, is_fixture=False, serial_channel=0, raw=True):
         self.stdin_fileno = sys.stdin.fileno()
         self.stdout_fileno = sys.stdout.fileno()
-        self.old_settings = termios.tcgetattr(self.stdin_fileno)
-        tty.setraw(self.stdin_fileno)
+        if raw:
+            self.old_settings = termios.tcgetattr(self.stdin_fileno)
+            tty.setraw(self.stdin_fileno)
+        else:
+            self.old_settings = None
         self.line_ending = line_ending
+        # self._opost = True if is_fixture else opost
         self._opost = opost
+        self._is_fixture = is_fixture
+        self._serial_channel = serial_channel
+        self.hdlc = HDLC(False)
 
     def shutdown(self):
         """
@@ -144,7 +153,8 @@ class TTYIO:
         self.restore()
 
     def restore(self):
-        termios.tcsetattr(self.stdin_fileno, termios.TCSADRAIN, self.old_settings)
+        if self.old_settings:
+            termios.tcsetattr(self.stdin_fileno, termios.TCSADRAIN, self.old_settings)
 
     def output(self, data, fg=None, flush=False):
         """
@@ -173,10 +183,76 @@ class TTYIO:
         if char == b'\x03':
             raise KeyboardInterrupt
 
+        if self._is_fixture:
+
+            UART_TX = 0x6A
+            output = bytearray()
+            output.append(UART_TX)
+            output.append(self._serial_channel)
+            output.append(1)
+            output.extend(char)
+            frame = self.hdlc.sendFrame(output)
+
+            os.write(self.stdout_fileno, char)
+            if char == b'\r':
+                os.write(self.stdout_fileno, self.line_ending)
+            sys.stdout.flush()
+
+            return frame
+
         if char == b'\r':
             return self.line_ending or char
+
         return char
 
+class PTTYIO:
+    """
+        Class for doing I/O with a TTY managed by curses
+    """
+    def __init__(self, line_ending=b'', opost=False, is_fixture=False):
+        self.stdin_fileno = sys.stdin.fileno()
+        self.stdout_fileno = sys.stdout.fileno()
+        # tty.setraw(self.stdin_fileno)
+        self.line_ending = line_ending
+        self.create_ptty()
+        self.stdin_poll = select.poll()
+        self.stdin_poll.register(self.stdin_fileno, select.POLLIN)
+        self.ttypoll = select.poll()
+        self.ttypoll.register(self.primary, select.POLLIN)
+
+    def create_ptty(self):
+        self.primary, self.secondary = pty.openpty()
+        self.s_name = os.ttyname(self.secondary)
+        click.echo(f"Opening a new tty device at {self.s_name}", nl=False)
+
+    def shutdown(self):
+        """
+            No special behavior needed
+        """
+        os.close(self.primary)
+        os.close(self.secondary)
+
+    def output(self, data, fg=None, flush=False):
+        """
+            Output some data to the TTY
+        """
+        os.write(self.stdout_fileno, data)
+
+    def read(self):
+        """
+            Read a key from the TTY
+        """
+        if self.stdin_poll.poll(0):
+            key_char = os.read(self.stdin_fileno, 1)
+            if key_char == b'\x03':
+                raise KeyboardInterrupt
+
+        if self.ttypoll.poll(0):
+            char = os.read(self.primary, 1)
+            if char == b'':
+                raise EOFError
+            return char
+
 class StandardIO:
     """
         Class for doing I/O with standard UNIX io streams
@@ -214,16 +290,21 @@ class StandardIO:
 
 
 @retry(reraise=True, sleep=trio.sleep, stop=stop_after_attempt(4), wait=wait_fixed(2), retry=retry_if_exception_type(trio_websocket.ConnectionRejected))
-async def display_job_output(connection_params, test_runner, interactive, line_ending, message_timeout, overall_timeout, eof_timeout, success_regex=None, failure_regex=None, opost=False):
+async def display_job_output(connection_params, test_runner, interactive, line_ending, message_timeout, overall_timeout, eof_timeout, success_regex=None, failure_regex=None, opost=False, serial_channel=0, ptty=False):
     """
         Display job output from websocket
     """
     (uri, kwargs) = connection_params
     match_class = test_matcher_factory(test_runner)
-    if interactive:
-        io_source = TTYIO(line_ending, opost)
+    is_fixture = test_runner == 'fixture'
+    
+    if ptty:
+        io_source = PTTYIO(line_ending, opost, is_fixture)
+    elif interactive:
+        io_source = TTYIO(line_ending, opost, is_fixture, serial_channel, raw=interactive=='raw')
     else:
         io_source = StandardIO()
+    
     try:
         matcher = match_class(io_source, success_regex, failure_regex)
         send_channel, receive_channel = trio.open_memory_channel(0)
@@ -245,7 +326,7 @@ async def display_job_output(connection_params, test_runner, interactive, line_e
         if io_source:
             io_source.shutdown()
 
-def run_job_output(connection_params, test_runner, interactive, line_ending, message_timeout, overall_timeout, eof_timeout, debug=False, success_regex=None, failure_regex=None, opost=False):
+def run_job_output(connection_params, test_runner, interactive, line_ending, message_timeout, overall_timeout, eof_timeout, debug=False, success_regex=None, failure_regex=None, opost=False, serial_channel=0, ptty=False):
     """
         Run async task to get job output from websocket
     """
@@ -264,7 +345,7 @@ def run_job_output(connection_params, test_runner, interactive, line_ending, mes
         raise ValueError('Invalid line ending')
 
     try:
-        matcher = trio.run(display_job_output, connection_params, test_runner, interactive, line_ending, message_timeout, overall_timeout, eof_timeout, success_regex, failure_regex, opost)
+        matcher = trio.run(display_job_output, connection_params, test_runner, interactive, line_ending, message_timeout, overall_timeout, eof_timeout, success_regex, failure_regex, opost, serial_channel, ptty)
         click.get_current_context().exit(matcher.exit_code)
     except trio.TooSlowError:
         suffix = '' if overall_timeout == 1 else 's'
